#!/usr/bin/env python

"""
cli.py - CLI specific functions for QuickRip.

copyright: (C) 2003, Tom Chance
license:   GNU General Public License (GPL) (see LICENSE file)
web:       http://quickrip.sf.net
email:     tomchance@gmx.net
"""
try:
	import sys, os, re, output
	from dvd import *
except ImportError, desc:
	print "Unable to load module (", desc, ")"
	raise SystemExit

# QuickRip global configuration.
import config   

__app__ = config.app
__author__ = config.author
__version__ = config.version
__date__ = config.date
__copyright__ = config.copyright
__license__ = config.license

class CLI(DVD):
	"""CLI-specific functions to add a CLI to the DVD class:
	main() - send user into various modes (entry, toplevel, track)
	askTrack() - from toplevel, select track to be edited
	changeTrack() - from track, select information to be edited
	changeName() - from track, change file name
	changeSize() - from track, change file size"""

	def main(self, mode="entry", track=None):
		os.system("clear")
		#print "QuickRip v0.6, (C) Tom Chance, 2003" 
		print output.bold("%s v%s, %s\n" % (__app__, __version__, __copyright__))
		if mode == 'entry':
			self.outdir = self.config['outputdir']
			self.volumead = 0
			self.aLanguage = "en"
			self.sLanguage = "None"
			print output.bold("\nHit enter to scan DVD")
			try:
				null = raw_input()
				del null
			except KeyboardInterrupt:
				print output.bold("\n\nExiting...")
				sys.exit(2)
			self.scanDVD()
			self.main("toplevel")

		elif mode == 'toplevel':
			print output.bold("\nTracks on DVD: ((*) means track will be ripped)")
			for track in self.tracks:
					star = "   "
					if track['rip'] == 'yes':
						star = "(*)"
					print "%2s Track %2s: %2s" % (star, track['id'], track['timelabel'])
			print ""
			print "(d) Output directory: \t%s" % (self.outdir)
			print "(v) Volume adjustment: \t%s" % (self.volumead)
			print "(a) Audio language: \t%s" % (self.aLanguage)
			print "(s) Subtitles language:\t%s" % (self.sLanguage)
			print ""
			print "(m) More settings"
			print "(q) Quit QuickRip"
			print ""
			self.askTrack()

		elif mode == 'settings':
			deinterlacing = self.dio[int(self.config['deinterlacing'])]
			aspectratio = self.aro[int(self.config['aspectratio'])]
			passes = self.config['passes']
			if self.config['videocodec'] == 1:
				codec = "XviD"
			else:
				codec = "DivX"
			if self.config['pdamode'] == 1:
				pda = "yes"
			else:
				pda = "no"
			if passes == 0:
				passes = 2
			print ""
			print output.bold("More Settings")
			print ""
			print "Paths:"
			print "(m) Mplayer:\t%s" % (self.config['mplayer'])
			print "(e) Mencoder:\t%s" % (self.config['mencoder'])
			print "(t) Tcprobe:\t%s" % (self.config['tcprobe'])
			print "(v) DVD Device\t%s" % (self.config['dvd_device'])
			print "(o) Default output dir:\t%s" % (self.config['outputdir'])
			print ""
			print "Mencoder options:"
			print "(d) De-Interlacing:\t%s" % (deinterlacing)
			print "(a) Aspect ration:\t%s" % (aspectratio)
			print "(p) Mencoder passes:\t%s" % (passes)
			print "(h) PDA mode:\t\t%s" % (pda)
			print "(c) Video codec:\t%s" % (codec)
			print ""
			print "(s) Save settings to config file"
			self.askSettings()

		elif mode == 'track':
			print ""
			print "Track %2s information:" % (track['id'])
			print "(n)  Name: \t\t%s" % (track['name'])
			print "     Length: \t\t%s" % (track['timelabel'])
			print "(s)  File size: \t%sMB" % (track['size'])
			print "(v)  Video bitrate: \t%skbps" % (track['vbr'])
			print "(a)  Audio bitrate: \t%skbps" % (track['abr'])
			print "(r)  Rip track? \t%s" % (track['rip'])
			self.changeTrack(track)

		elif mode == 'rip':
			print ""
			print output.bold("* QuickRip will rip these DVD tracks:")
			trackno = 0
			for track in self.tracks:
				if track['rip'] == 'yes':
					trackno = trackno + 1
					print "Track %2s (%s/%s.avi) at %skbps" % (track['id'], self.outdir, track['name'], track['vbr'])
			if trackno == 0:
				print output.bold("You don't seem to have selected any tracks to rip!")
				print output.bold("Hit enter to return to the main menu")
				null = raw_input()
				del null
				self.main("toplevel")
			print output.bold("Hit enter to begin ripping, or 'b' to return to the menu")
			null = raw_input("> ")
			if null == 'b':
				self.main("toplevel")
			del null
			self.outdir = self.outdir + "/"
			self.ripDVD()
			self.main("toplevel")
    
	def askTrack(self):
		print output.bold("Enter letter/track number to edit information, or 'r' to start ripping")
		try:
			choice = raw_input("> ")
		except KeyboardInterrupt:
			print "\n\nExiting..."
			sys.exit(2)
        
		if choice == 'r':
			self.main("rip")
		elif choice == 'd':
			self.changeDir()
		elif choice == 'v':
			self.changeVolume()
		elif choice == 'a':
			self.changeALang()
		elif choice == 's':
			self.changeSLang()
		elif choice == 'm':
			self.main("settings")
		elif choice == 'q':
			self.quit()
        
		try:
			choice = int(choice)
			track = self.tracks[choice - 1]
		except:
			print output.bold("Invalid track number!")
			self.askTrack()
        	self.main("track", track)

	def changeDir(self):
		#print ""
		print output.bold("\nEnter new output directory:")
		path = raw_input("> ")
		if os.path.exists(path) == 0:
			print output.bold("Invalid directory!")
			self.changeDir()
		else:
			self.outdir = path
		self.main("toplevel")
    
	def changeVolume(self):
		#print ""
		print output.bold("\nEnter new volume adjustment (0-10):")
		vol = raw_input("> ")
		try:
			vol = int(vol)
		except:
			print output.bold("Invalid volume adjustment!")
			self.changeVolume()
		if vol < 0 or vol > 10:
			print output.bold("Invalid volume adjustment!")
			self.changeVolume()         
		self.volumead = vol
		self.main("toplevel")
    
	def changeALang(self):
		langs = ""
		for lang in self.alangs:
			langs = langs + lang + " "
		#print ""
		print output.bold("\nEnter audio language (%s)" % (langs))
		langchoice = raw_input("> ")
		success = 0
		for lang in self.alangs:
			if langchoice == lang:
				self.aLanguage = langchoice
				success = 1
		if success == 0:
			print output.bold("Invalid language!")
			self.changeALang()
		del success, lang, langchoice
		self.main("toplevel")
    
	def changeSLang(self):
		lang = ""
		langs = "None "
		for lang in self.slangs:
			langs = langs + lang + " "
		#print ""
		print output.bold("\nEnter subtitle language (%s)" % (langs))
		langchoice = raw_input("> ")
		success = 0
		for lang in self.slangs:
			if langchoice == lang:
				self.sLanguage = langchoice
				success = 1
		if langchoice == "None":
			self.sLanguage = langchoice
			success = 1
		if success == 0:
			print output.bold("Invalid language!")
			self.changeSLang()
		del success, lang, langchoice
		self.main("toplevel")

	def quit(self):
		os.system("clear")
		sys.exit(2)
    
	def askSettings(self):
		#print ""
		print output.bold("\nEnter letter to change setting, or 'b' to go back to the track listing")
		choice = raw_input("> ")

		if choice == 'm':
			self.changePath("mplayer")
		elif choice == 'e':
			self.changePath("mencoder")
		elif choice == 't':
			self.changePath("tcprobe")
		elif choice == 'v':
			self.changePath("dvd_device")
		elif choice == 'o':
			self.changePath("outputdir")
		elif choice == 'd':
			self.changeDeinterlacing()
		elif choice == 'a':
			self.changeAspectRatio()
		elif choice == 'p':
			self.changePasses()
		elif choice == 'h':
			self.changePDAMode()
		elif choice == 'c':
			self.changeVideoCodec()
		elif choice == 's':
			self.saveSettings()
		elif choice == 'b':
			self.main("toplevel")
		else:
			print output.bold("Invalid operation!")
			self.askSettings()
    
	def changePath(self, type):
		#print ""
		print output.bold("\nEnter new path for %s:" % (type))
		path = raw_input("> ")
		if not os.path.exists(path):
			print output.bold("Invalid path!")
			self.changePath(type)
		self.config[type] = path
		self.main("settings")
    
	def changeDeinterlacing(self):
		#print ""
		print output.bold("\nEnter interlacing option:")
		for key in self.dio.keys():
			print "(%s) %s" % (str(key), self.dio[key])
		print ""
		choice = raw_input("> ")
		choice = int(choice)
		try:
			null = self.dio[choice]
		except:
			print output.bold("Invalid option!")
		self.changeDeinterlacing()
		self.config['deinterlacing'] = choice
		self.main("settings")
    
	def changeAspectRatio(self):
		#print ""
		print output.bold("\nEnter aspect ratio:")
		for key in self.aro.keys():
			print "(%s) %s" % (str(key), self.aro[key])
		print ""
		try:
			choice = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		try:
			choice = int(choice)
		except ValueError, error_message:
			print output.bold("Invalid value:", error_message)
		try:
			null = self.aro[choice]
		except:
			print output.bold("Invalid ratio!")
			self.changeAspectRatio()
		self.config['aspectratio'] = choice
		self.main("settings")
    
	def changePasses(self):
		#print ""
		print output.bold("\nEnter number of passes (see online documentation) (1, 2 or 3):")
		try:
			choice = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		try:
			choice = int(choice)
		except ValueError, error_message:
			print output.bold("Invalid value:", error_message)
		if choice < 1 or choice > 3:
			print output.bold("Invalid number!")
			self.changePasses()
		self.config['passes'] = choice
		self.main("settings")

	def changePDAMode(self):
		print output.bold("\nPDA Mode: (yes/no):")
		try:
			choice = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		if choice not in ['yes', 'no']:
			print output.bold("Invalid option!")
			self.changePDAMode()
		if choice == 'yes':
			self.config['pdamode'] = 1
		else:
			self.config['pdamode'] = 0
		self.main("settings")

	def changeVideoCodec(self):
		print output.bold("\nVideo Codec: (0 = DivX, 1 = XviD):")
		try:
			choice = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		if choice not in ['0', '1']:
			print output.bold("Invalid option!")
			self.changeVideoCodec()
		self.config['videocodec'] = int(choice)
		self.main("settings")

    
	def saveSettings(self):
		#print ""
		print "\n* Saving settings to %s" % (self.configfile)
		self.parser.set('paths', 'mplayer', self.config['mplayer'])
		self.parser.set('paths', 'mencoder', self.config['mencoder'])
		self.parser.set('paths', 'tcprobe', self.config['tcprobe'])
		self.parser.set('paths', 'outputdir', self.config['outputdir'])
		self.parser.set('paths', 'dvd_device', self.config['dvd_device'])
		self.parser.set('mencoder', 'deinterlacing', self.config['deinterlacing'])
		self.parser.set('mencoder', 'aspectratio', self.config['aspectratio'])
		self.parser.set('mencoder', 'passes', self.config['passes'])
		self.parser.set('mencoder', 'pdamode', self.config['pdamode'])
		self.parser.set('mencoder', 'videocodec', self.config['videocodec'])
		self.parser.write(open(self.configfile, 'w'))
		print "* Completed"
		print output.bold("Press enter to continue")
		try:
			null = raw_input()
			del null
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		self.main("settings")
    

	def changeTrack(self, track):
		#print ""
		print output.bold("\nEnter letter to change information, or 'b' to go back to the track listing")
		try:
			choice = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
        
		if choice == 'n':
			self.changeName(track)
		elif choice == 's':
			self.changeSize(track)
		elif choice == 'v':
			self.changeVBR(track)
		elif choice == 'a':
			self.changeABR(track)
		elif choice == 'r':
			self.changeRip(track)
		elif choice == 'b':
			self.main("toplevel")
		else:
			print output.bold("Invalid operation!")
			self.changeTrack(track)
        
	def changeName(self, track):
		#print ""
		print output.bold("\nEnter new name:")
		try:
			name = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		track['name'] = name
		self.main("track", track)
    
	def changeSize(self, track):
        #print ""
		print output.bold("\nEnter new file size (MB):")
		try:
			size = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		try:
			size = int(size)
		except ValueError:
			print output.bold("Invalid size!")
			self.changeSize(track)
		track['size'] = size
		track['vbr'] = self.calcRate(track['time'], track['abr'], size)
		self.main("track", track)

	def changeVBR(self, track):
        #print ""
		print output.bold("\nEnter new video bitrate (kbps):")
		try:
			vbr = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		try:
			vbr = int(vbr)
		except:
			print output.bold("Invalid bitrate!")
			self.changeVBR(track)
		track['vbr'] = vbr
		track['size'] = int(self.calcFileSize(track['time'], track['abr'], track['vbr']))
		self.main("track", track)

	def changeABR(self, track):
        #print ""
		print output.bold("\nEnter new audio bitrate (kbps) [64, 96, 128, 160]:")
		try:
			abr = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		try:
			abr = int(abr)
		except:
			print output.bold("Invalid bitrate!")
			self.changeABR(track)
		if abr != 64 and abr != 96 and abr != 128 and abr != 160:
			print output.bold("Invalid bitrate!")
			self.changeABR(track)
		track['abr'] = abr
		track['vbr'] = self.calcRate(track['time'], abr, track['size'])
		self.main("track", track)

	def changeRip(self, track):
		#print ""
		print output.bold("\nRip track? [yes/no]:")
		try:
			rip = raw_input("> ")
		except KeyboardInterrupt:
			print output.bold("\n\nExiting...")
			sys.exit(2)
		if rip not in ['yes', 'no']:
			print output.bold("Invalid option!")
			self.changeRip(track)
		track['rip'] = rip
		self.main("track", track)

	def int_configError(self, bin):
		print output.bold("Configuration error: Unable to find %s, check it is installed" % (bin))
		sys.exit(2)

	def int_startScanning(self):
		print "* Scanning DVD..."

	def int_noTracks(self):
		#print "*** ERROR: No DVD tracks found ***"
		#print "*** Check:"
		#print "***    - Mplayer and transcode are installed"
		#print "***    - Mplayer can play DVDs"
		#print "***    - There is a DVD in the drive"
		print """
		*** ERROR: No DVD tracks found ***
		*** Check:
		***    - Mplayer and transcode are installed
		***    - Mplayer can play DVDs
		***    - There is a DVD in the drive
		"""
		sys.exit(2)

	def int_dispDVD(self):
		print "Found %s titles" % (self.numtitles)
		print "Found audio languages %s" % (self.alangs)
		print "Found subtitle languages %s" % (self.slangs)

	def int_dispTrack(self, track):
		self.track = track
		print "Track %2s: %s" % (self.track['id'], self.track['timelabel'])

	def int_finishScanning(self):
		print "* Finished scanning"

	def int_startRipping(self):
		#print ""
		print "\n* Ripping started"

	def int_newTrack(self, name, number, total, vbr):
		#print ""
		print "\n* Ripping track %2s (%s of %s)" % (name, number, total)

	def int_newPass(self, passtype):
		slashn = ""
		if self.config['passes'] == 1:
			passtype = "Progress"
		elif self.config['passes'] == 2 and passtype == "video1":
			passtype = "video"
		if passtype != "audio" and passtype != "Progress":
			sys.stdout.write(" 100% ")
			for i in range(self.n):
				sys.stdout.write("=")
			sys.stdout.write("|\n")
		sys.stdout.write(slashn + passtype + " |")
        
		self.spinner = "\|/-\|/-"
		self.spinpos = 0
		width = os.popen("tput cols").read()
		width = float(width)
		self.jumper = 100 / (width - 20)
		self.n = 0

	def int_updateProgress(self, perc, trem, tpass):
		for opt in sys.argv:
			if opt in ['-a', '--autoresize']:
				width = os.popen("tput cols").read()
				width = float(width)
				self.jumper = 100 / (width - 20)
    
        	n = int(int(perc) / self.jumper)

		if int(perc) <= 9:
			sys.stdout.write("  " + str(int(perc)) + "% |")
		else:
			sys.stdout.write(" " + str(int(perc)) + "% |")

		sys.stdout.write("=" * n)

		sys.stdout.write(self.spinner[self.spinpos])
		self.spinpos=(self.spinpos+1)%8
		sys.stdout.flush()

		sys.stdout.write("\b" * (7+n))

	def int_finishRipping(self):
		#print ""
		print "\n* Ripping finished\nPress enter to return to main menu"
		try:
			null = raw_input()
			del null
		except KeyboardInterrupt:
			print "\n\nExiting..."
			sys.exit(2)
		self.main("toplevel")

def main():
	#print "QuickRip v0.6, (C) Tom Chance, 2003"
	print "%s v%s, %s\n" % (__app__, __version__, __copyright__)    
	cli = CLI()
	cli.main()
