#!/usr/bin/env python

"""
quickrip.py - Main program file for QuickRip.

copyright: (C) 2003, Tom Chance
license:   GNU General Public License (GPL) (see LICENSE file)
web:       http://www.tomchance.uklinux.net/projects/quickrip.shtml
email:     tomchance@gmx.net
"""

import sys
import os
import getopt

# QuickRip global configuration.
import config   

__app__ = config.app
__author__ = config.author
__version__ = config.version
__date__ = config.date
__copyright__ = config.copyright
__license__ = config.license

__doc__ = """
Usage: %s [options]

Options:  (with none, runs PyQt GUI mode)
    -h, --help              print this help message
    -c, --cli               trun in command line interface mode
    -a, --autoresize        automatically resize the cli progress bar as
                            the terminal is resized (small performance hit)
""" % sys.argv[0]

def usage():
    print "%s v%s, %s\n%s" % (__app__, __version__, __copyright__, __doc__)

def main(argv):
    try:
        opts, args = getopt.getopt(argv, "hgca", ["help", "gtk", "cli", "autoresize"])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
	elif opt in ("-g", "--gtk"):
	    sys.path.append("gtk")
	    import gtkmain
	    gtkmain.main()
        elif opt in ("-c", "--cli"):
            import cli
            cli.main()
        elif opt in ("-a", "--autoresize"):
            # Help!  What do I do here???
            pass

    # Fallback to GUI mode...
    if 'DISPLAY' in os.environ:
        import gui
        gui.main()
    else:
        print "No $DISPLAY found.  Using CLI interface"
        import cli
        cli.main()

# The famous __name__ + __main__ trick...
if __name__ == '__main__':
    main(sys.argv[1:])
